# -*- coding: utf-8 -*-

"""
apimaticapi

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimaticapi.api_helper import APIHelper
from apimaticapi.configuration import Server
from apimaticapi.utilities.file_wrapper import FileWrapper
from apimaticapi.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from apimaticapi.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.configurations.endpoint_configuration import EndpointConfiguration
from apimaticapi.models.user_code_generation import UserCodeGeneration


class CodeGenerationExternalApisController(BaseController):

    """A Controller to access Endpoints in the apimaticapi API."""
    def __init__(self, config):
        super(CodeGenerationExternalApisController, self).__init__(config)

    def generate_sdk_via_file(self,
                              file,
                              template):
        """Does a POST request to /code-generations/generate-via-file.

        Generate an SDK for an API by by uploading the API specification file.
                This endpoint generates and then uploads the generated SDK to
        APIMatic's cloud storage. An ID for the generation performed is
        returned as part of the response.
        This endpoint does not import an API into APIMatic.

        Args:
            file (typing.BinaryIO): The API specification file.<br>The type of
                the specification file should be any of the [supported
                formats](https://docs.apimatic.io/api-transformer/overview-tran
                sformer#supported-input-formats).
            template (Platforms): The structure contains platforms that
                APIMatic CodeGen can generate SDKs and Docs in.

        Returns:
            UserCodeGeneration: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/code-generations/generate-via-file')
            .http_method(HttpMethodEnum.POST)
            .multipart_param(Parameter()
                             .key('file')
                             .value(file)
                             .default_content_type('application/octet-stream'))
            .form_param(Parameter()
                        .key('template')
                        .value(template))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Authorization'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(UserCodeGeneration.from_dictionary)
        ).execute()

    def generate_sdk_via_url(self,
                             body):
        """Does a POST request to /code-generations/generate-via-url.

        Generate an SDK for an API by providing the URL of the API
        specification file. 
        This endpoint generates and then uploads the generated SDK to
        APIMatic's cloud storage. An ID for the generation performed is
        returned as part of the response.
        This endpoint does not import an API into APIMatic.

        Args:
            body (GenerateSdkViaUrlRequest): Request Body

        Returns:
            UserCodeGeneration: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/code-generations/generate-via-url')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/vnd.apimatic.userCodeGenerationDto.v1+json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Authorization'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(UserCodeGeneration.from_dictionary)
        ).execute()

    def download_generated_sdk(self,
                               codegen_id):
        """Does a GET request to /code-generations/{codegen_id}/generated-sdk.

        Download the SDK generated via the Generate SDK endpoints.

        Args:
            codegen_id (str): The ID of code generation received in the
                response of the [Generate SDK Via
                File](https://www.apimatic.io/api-docs-preview/dashboard/60eea3
                b7a73395c3052d961b/v/3_0#/http/api-endpoints/code-generation-ex
                ternal-apis/generate-sdk-via-file) or [Generate SDK Via URL
                ](https://www.apimatic.io/api-docs-preview/dashboard/60eea3b7a7
                3395c3052d961b/v/3_0#/http/api-endpoints/code-generation-extern
                al-apis/generate-sdk-via-url) calls.

        Returns:
            binary: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/code-generations/{codegen_id}/generated-sdk')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('codegen_id')
                            .value(codegen_id)
                            .should_encode(True))
            .auth(Single('Authorization'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
        ).endpoint_configuration(
            EndpointConfiguration()
            .has_binary_response(True)
        ).execute()

    def list_all_code_generations_external(self):
        """Does a GET request to /code-generations.

        Get a list of all SDK generations performed with external APIs via the
        Generate SDK endpoints.

        Returns:
            List[UserCodeGeneration]: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/code-generations')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Authorization'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(UserCodeGeneration.from_dictionary)
        ).execute()

    def download_input_file_codegen(self,
                                    codegen_id):
        """Does a GET request to /code-generations/{codegen_id}/input-file.

        Download the API Specification file used as input for a specific SDK
        generation performed via the Generate SDK endpoints.

        Args:
            codegen_id (str): The ID of the code generation to download the
                API specification for. The code generation ID is received in
                the response of the [Generate SDK Via
                File](https://www.apimatic.io/api-docs-preview/dashboard/60eea3
                b7a73395c3052d961b/v/3_0#/http/api-endpoints/code-generation-ex
                ternal-apis/generate-sdk-via-file) or [Generate SDK Via URL
                ](https://www.apimatic.io/api-docs-preview/dashboard/60eea3b7a7
                3395c3052d961b/v/3_0#/http/api-endpoints/code-generation-extern
                al-apis/generate-sdk-via-url) calls

        Returns:
            binary: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/code-generations/{codegen_id}/input-file')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('codegen_id')
                            .value(codegen_id)
                            .should_encode(True))
            .auth(Single('Authorization'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
        ).endpoint_configuration(
            EndpointConfiguration()
            .has_binary_response(True)
        ).execute()

    def get_a_code_generation_codegen(self,
                                      codegen_id):
        """Does a GET request to /code-generations/{codegen_id}.

        Get details on an SDK generation performed for an external API via the
        Generate SDK endpoints.

        Args:
            codegen_id (str): The ID of the code generation to fetch. The code
                generation ID is received in the response of the [Generate SDK
                Via
                File](https://www.apimatic.io/api-docs-preview/dashboard/60eea3
                b7a73395c3052d961b/v/3_0#/http/api-endpoints/code-generation-ex
                ternal-apis/generate-sdk-via-file) or [Generate SDK Via URL
                ](https://www.apimatic.io/api-docs-preview/dashboard/60eea3b7a7
                3395c3052d961b/v/3_0#/http/api-endpoints/code-generation-extern
                al-apis/generate-sdk-via-url) calls.

        Returns:
            UserCodeGeneration: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/code-generations/{codegen_id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('codegen_id')
                            .value(codegen_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Authorization'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(UserCodeGeneration.from_dictionary)
        ).execute()

    def delete_code_generation_1(self,
                                 codegen_id):
        """Does a DELETE request to /code-generations/{codegen_id}.

        Delete an SDK generation performed for an API via the Generate SDK
        endpoints.

        Args:
            codegen_id (str): The ID of the code generation to delete. The
                code generation ID is received in the response of the
                [Generate SDK Via
                File](https://www.apimatic.io/api-docs-preview/dashboard/60eea3
                b7a73395c3052d961b/v/3_0#/http/api-endpoints/code-generation-ex
                ternal-apis/generate-sdk-via-file) or [Generate SDK Via URL
                ](https://www.apimatic.io/api-docs-preview/dashboard/60eea3b7a7
                3395c3052d961b/v/3_0#/http/api-endpoints/code-generation-extern
                al-apis/generate-sdk-via-url) calls.

        Returns:
            void: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/code-generations/{codegen_id}')
            .http_method(HttpMethodEnum.DELETE)
            .template_param(Parameter()
                            .key('codegen_id')
                            .value(codegen_id)
                            .should_encode(True))
            .auth(Single('Authorization'))
        ).execute()
